@@ -574,7 +574,7 @@ int is_msg_for_me(snac *snac, const xs_dict *c_msg)
         return 0;
     }
 
-    if (xs_match(type, "Like|Announce")) {
+    if (xs_match(type, "Like|Announce|EmojiReact")) {
         const char *object = xs_dict_get(c_msg, "object");
 
         if (xs_type(object) == XSTYPE_DICT)
@@ -834,7 +834,7 @@ void notify(snac *snac, const char *type, const char *utype, const char *actor,
     if (xs_type(objid) == XSTYPE_DICT)
         objid = xs_dict_get(objid, "id");
 
-    if (strcmp(type, "Like") == 0 || strcmp(type, "Announce") == 0) {
+    if (xs_match(type, "Like|Announce|EmojiReact")) {
         /* if it's not an admiration about something by us, done */
         if (xs_is_null(objid) || !xs_startswith(objid, snac->actor))
             return;
@@ -949,7 +949,7 @@ void notify(snac *snac, const char *type, const char *utype, const char *actor,
     if (strcmp(utype, "Follow") == 0)
         objid = actor;
 
-    notify_add(snac, type, utype, actor, objid != NULL ? objid : id);
+    notify_add(snac, type, utype, actor, objid != NULL ? objid : id, msg);
 }
 
 /** messages **/
@@ -1901,10 +1901,10 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
             }
         }
         else
-        if (strcmp(utype, "Like") == 0) { /** **/
+        if (strcmp(utype, "Like") == 0 || strcmp(utype, "EmojiReact") == 0) { /** **/
             int status = object_unadmire(id, actor, 1);
 
-            snac_log(snac, xs_fmt("Unlike for %s %d", id, status));
+            snac_log(snac, xs_fmt("Undo '%s' for %s %d", utype, id, status));
         }
         else
         if (strcmp(utype, "Announce") == 0) { /** **/
@@ -2014,12 +2014,12 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
         }
     }
     else
-    if (strcmp(type, "Like") == 0) { /** **/
+    if (strcmp(type, "Like") == 0 || strcmp(type, "EmojiReact") == 0) { /** **/
         if (xs_type(object) == XSTYPE_DICT)
             object = xs_dict_get(object, "id");
 
         if (timeline_admire(snac, object, actor, 1) == HTTP_STATUS_CREATED)
-            snac_log(snac, xs_fmt("new 'Like' %s %s", actor, object));
+            snac_log(snac, xs_fmt("new '%s' %s %s", type, actor, object));
         else
             snac_log(snac, xs_fmt("repeated 'Like' from %s to %s", actor, object));
 
@@ -2134,6 +2134,73 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
 
         enqueue_output_by_actor(snac, rsp, actor, 0);
     }
+    else
+    if (strcmp(type, "Block") == 0) { /** **/
+        snac_log(snac, xs_fmt("'Block' received from %s", actor));
+
+        /* should we MUTE the actor back? */
+        /* mute(snac, actor); */
+
+        do_notify = 1;
+    }
+    else
+    if (strcmp(type, "Move") == 0) { /** **/
+        do_notify = 1;
+
+        const char *old_account = xs_dict_get(msg, "object");
+        const char *new_account = xs_dict_get(msg, "target");
+
+        if (!xs_is_null(old_account) && !xs_is_null(new_account)) {
+            if (following_check(snac, old_account)) {
+                xs *n_actor = NULL;
+
+                if (valid_status(object_get(new_account, &n_actor))) {
+                    const xs_list *aka = xs_dict_get(n_actor, "alsoKnownAs");
+
+                    if (xs_type(aka) == XSTYPE_LIST) {
+                        if (xs_list_in(aka, old_account) != -1) {
+                            /* all conditions met! */
+
+                            /* follow new account */
+                            xs *f_msg = msg_follow(snac, new_account);
+
+                            if (f_msg != NULL) {
+                                const char *new_actor = xs_dict_get(f_msg, "object");
+                                following_add(snac, new_actor, f_msg);
+                                enqueue_output_by_actor(snac, f_msg, new_actor, 0);
+
+                                snac_log(snac, xs_fmt("'Move': following %s", new_account));
+                            }
+
+                            /* unfollow old account */
+                            xs *of_msg = NULL;
+
+                            if (valid_status(following_get(snac, old_account, &of_msg))) {
+                                xs *uf_msg = msg_undo(snac, xs_dict_get(of_msg, "object"));
+                                following_del(snac, old_account);
+                                enqueue_output_by_actor(snac, uf_msg, old_account, 0);
+
+                                snac_log(snac, xs_fmt("'Move': unfollowing %s", old_account));
+                            }
+                        }
+                        else
+                            snac_log(snac, xs_fmt("'Move' error: old actor %s not found in %s 'alsoKnownAs'",
+                                old_account, new_account));
+                    }
+                    else
+                        snac_log(snac, xs_fmt("'Move' error: cannot get %s 'alsoKnownAs'", new_account));
+                }
+                else
+                    snac_log(snac, xs_fmt("'Move' error: cannot get new actor %s", new_account));
+            }
+            else
+                snac_log(snac, xs_fmt("'Move' error: actor %s is not being followed", old_account));
+        }
+        else {
+            snac_log(snac, xs_fmt("'Move' error: malformed message from %s", actor));
+            srv_archive_error("move", "move", req, msg);
+        }
+    }
     else {
         srv_archive_error("unsupported_type", "unsupported_type", req, msg);
 
@@ -2394,6 +2461,7 @@ void process_queue_item(xs_dict *q_item)
                 || status == HTTP_STATUS_NOT_FOUND
                 || status == HTTP_STATUS_METHOD_NOT_ALLOWED
                 || status == HTTP_STATUS_GONE
+                || status == HTTP_STATUS_UNPROCESSABLE_CONTENT
                 || status < 0)
                 /* explicit error: discard */
                 srv_log(xs_fmt("output message: fatal error %s %d", inbox, status));
