@@ -88,9 +88,11 @@ static xs_str *format_line(const char *line, xs_list **attach)
     /* split by markup */
     xs *sm = xs_regex_split(line,
         "("
-            "`[^`]+`"                   "|"
-            "\\*\\*?[^\\*]+\\*?\\*"     "|"
-            "\\[[^]]+\\]\\([^\\)]+\\)"  "|"
+            "`[^`]+`"                           "|"
+            "~~[^~]+~~"                         "|"
+            "\\*\\*?\\*?[^\\*]+\\*?\\*?\\*"     "|"
+            "!\\[[^]]+\\]\\([^\\)]+\\)"         "|"
+            "\\[[^]]+\\]\\([^\\)]+\\)"          "|"
             "https?:/" "/[^[:space:]]+"
         ")");
     int n = 0;
@@ -100,24 +102,37 @@ static xs_str *format_line(const char *line, xs_list **attach)
         if ((n & 0x1)) {
             /* markup */
             if (xs_startswith(v, "`")) {
-                xs *s1 = xs_crop_i(xs_dup(v), 1, -1);
+                xs *s1 = xs_strip_chars_i(xs_dup(v), "`");
                 xs *e1 = encode_html(s1);
                 xs *s2 = xs_fmt("<code>%s</code>", e1);
                 s = xs_str_cat(s, s2);
             }
             else
+            if (xs_startswith(v, "***")) {
+                xs *s1 = xs_strip_chars_i(xs_dup(v), "*");
+                xs *s2 = xs_fmt("<b><i>%s</i></b>", s1);
+                s = xs_str_cat(s, s2);
+            }
+            else
             if (xs_startswith(v, "**")) {
-                xs *s1 = xs_crop_i(xs_dup(v), 2, -2);
+                xs *s1 = xs_strip_chars_i(xs_dup(v), "*");
                 xs *s2 = xs_fmt("<b>%s</b>", s1);
                 s = xs_str_cat(s, s2);
             }
             else
             if (xs_startswith(v, "*")) {
-                xs *s1 = xs_crop_i(xs_dup(v), 1, -1);
+                xs *s1 = xs_strip_chars_i(xs_dup(v), "*");
                 xs *s2 = xs_fmt("<i>%s</i>", s1);
                 s = xs_str_cat(s, s2);
             }
             else
+            if (xs_startswith(v, "~~")) {
+                xs *s1 = xs_strip_chars_i(xs_dup(v), "~");
+                xs *e1 = encode_html(s1);
+                xs *s2 = xs_fmt("<s>%s</s>", e1);
+                s = xs_str_cat(s, s2);
+            }
+            else
             if (xs_startswith(v, "http")) {
                 xs *u  = xs_replace(v, "#", "&#35;");
                 xs *v2 = xs_strip_chars_i(xs_dup(u), ".,)");
@@ -143,8 +158,8 @@ static xs_str *format_line(const char *line, xs_list **attach)
             else
             if (*v == '[') {
                 /* markdown-like links [label](url) */
-                xs *w    = xs_strip_chars_i(xs_dup(v), "[)");
-                xs *l    = xs_split_n(w, "](", 1);
+                xs *w = xs_strip_chars_i(xs_replace(v, "#", "&#35;"), "[)");
+                xs *l = xs_split_n(w, "](", 1);
 
                 if (xs_list_len(l) == 2) {
                     xs *link = xs_fmt("<a href=\"%s\">%s</a>",
@@ -155,6 +170,36 @@ static xs_str *format_line(const char *line, xs_list **attach)
                 else
                     s = xs_str_cat(s, v);
             }
+            else
+            if (*v == '!') {
+                /* markdown-like images ![alt text](url to image) */
+                xs *w = xs_strip_chars_i(xs_replace(v, "#", "&#35;"), "![)");
+                xs *l = xs_split_n(w, "](", 1);
+
+                if (xs_list_len(l) == 2) {
+                    const char *alt_text = xs_list_get(l, 0);
+                    const char *img_url  = xs_list_get(l, 1);
+                    const char *mime     = xs_mime_by_ext(img_url);
+
+                    if (attach != NULL && xs_startswith(mime, "image/")) {
+                        xs *d = xs_dict_new();
+
+                        d = xs_dict_append(d, "mediaType", mime);
+                        d = xs_dict_append(d, "url",       img_url);
+                        d = xs_dict_append(d, "name",      alt_text);
+                        d = xs_dict_append(d, "type",      "Image");
+
+                        *attach = xs_list_append(*attach, d);
+                    }
+                    else {
+                        xs *link = xs_fmt("<a href=\"%s\">%s</a>", img_url, alt_text);
+
+                        s = xs_str_cat(s, link);
+                    }
+                }
+                else
+                    s = xs_str_cat(s, v);
+            }
             else
                 s = xs_str_cat(s, v);
         }
