@@ -5,6 +5,7 @@
 #include "xs_io.h"
 #include "xs_json.h"
 #include "xs_socket.h"
+#include "xs_unix_socket.h"
 #include "xs_httpd.h"
 #include "xs_mime.h"
 #include "xs_time.h"
@@ -761,8 +762,9 @@ srv_state *srv_state_op(xs_str **fname, int op)
 void httpd(void)
 /* starts the server */
 {
-    const char *address;
-    const char *port;
+    const char *address = NULL;
+    const char *port = NULL;
+    xs *full_address = NULL;
     int rs;
     pthread_t threads[MAX_THREADS] = {0};
     int n;
@@ -771,10 +773,20 @@ void httpd(void)
     sem_t anon_job_sem;
 
     address = xs_dict_get(srv_config, "address");
-    port    = xs_number_str(xs_dict_get(srv_config, "port"));
 
-    if ((rs = xs_socket_server(address, port)) == -1) {
-        srv_log(xs_fmt("cannot bind socket to %s:%s", address, port));
+    if (*address == '/') {
+        rs = xs_unix_socket_server(address, NULL);
+        full_address = xs_fmt("unix:%s", address);
+    }
+    else {
+        port = xs_number_str(xs_dict_get(srv_config, "port"));
+        full_address = xs_fmt("%s:%s", address, port);
+
+        rs = xs_socket_server(address, port);
+    }
+
+    if (rs == -1) {
+        srv_log(xs_fmt("cannot bind socket to %s", full_address));
         return;
     }
 
@@ -791,13 +803,13 @@ void httpd(void)
     signal(SIGTERM, term_handler);
     signal(SIGINT,  term_handler);
 
-    srv_log(xs_fmt("httpd%s start %s:%s %s", p_state->use_fcgi ? " (FastCGI)" : "",
-                    address, port, USER_AGENT));
+    srv_log(xs_fmt("httpd%s start %s %s", p_state->use_fcgi ? " (FastCGI)" : "",
+                    full_address, USER_AGENT));
 
     /* show the number of usable file descriptors */
     struct rlimit r;
     getrlimit(RLIMIT_NOFILE, &r);
-    srv_debug(0, xs_fmt("available (rlimit) fds: %d (cur) / %d (max)",
+    srv_debug(1, xs_fmt("available (rlimit) fds: %d (cur) / %d (max)",
                         (int) r.rlim_cur, (int) r.rlim_max));
 
     /* initialize the job control engine */
@@ -847,9 +859,10 @@ void httpd(void)
 
     if (setjmp(on_break) == 0) {
         for (;;) {
-            FILE *f = xs_socket_accept(rs);
+            int cs = xs_socket_accept(rs);
 
-            if (f != NULL) {
+            if (cs != -1) {
+                FILE *f = fdopen(cs, "r+");
                 xs *job = xs_data_new(&f, sizeof(FILE *));
                 job_post(job, 1);
             }
@@ -875,7 +888,7 @@ void httpd(void)
 
     xs *uptime = xs_str_time_diff(time(NULL) - p_state->srv_start_time);
 
-    srv_log(xs_fmt("httpd%s stop %s:%s (run time: %s)",
+    srv_log(xs_fmt("httpd%s stop %s (run time: %s)",
                 p_state->use_fcgi ? " (FastCGI)" : "",
-                address, port, uptime));
+                full_address, uptime));
 }
