@@ -1,7 +1,7 @@
 /* snac - A simple, minimalistic ActivityPub instance */
 /* copyright (c) 2022 - 2024 grunfink et al. / MIT license */
 
-#define VERSION "2.54"
+#define VERSION "2.57"
 
 #define USER_AGENT "snac/" VERSION
 
@@ -20,6 +20,8 @@
 #define MAX_CONVERSATION_LEVELS 48
 #endif
 
+#define MD5_HEX_SIZE 33
+
 extern double disk_layout;
 extern xs_str *srv_basedir;
 extern xs_dict *srv_config;
@@ -98,9 +100,11 @@ double f_ctime(const char *fn);
 int index_add_md5(const char *fn, const char *md5);
 int index_add(const char *fn, const char *id);
 int index_gc(const char *fn);
-int index_first(const char *fn, char *buf, int size);
+int index_first(const char *fn, char md5[MD5_HEX_SIZE]);
 int index_len(const char *fn);
 xs_list *index_list(const char *fn, int max);
+int index_desc_next(FILE *f, char md5[MD5_HEX_SIZE]);
+int index_desc_first(FILE *f, char md5[MD5_HEX_SIZE], int skip);
 xs_list *index_list_desc(const char *fn, int skip, int show);
 
 int object_add(const char *id, const xs_dict *obj);
@@ -126,7 +130,7 @@ int object_announces_len(const char *id);
 xs_list *object_children(const char *id);
 xs_list *object_likes(const char *id);
 xs_list *object_announces(const char *id);
-int object_parent(const char *id, char *buf, int size);
+int object_parent(const char *md5, char parent[MD5_HEX_SIZE]);
 
 int object_user_cache_add(snac *snac, const char *id, const char *cachedir);
 int object_user_cache_del(snac *snac, const char *id, const char *cachedir);
@@ -203,7 +207,7 @@ void lastlog_write(snac *snac, const char *source);
 
 xs_str *notify_check_time(snac *snac, int reset);
 void notify_add(snac *snac, const char *type, const char *utype,
-                const char *actor, const char *objid);
+                const char *actor, const char *objid, const xs_dict *msg);
 xs_dict *notify_get(snac *snac, const char *id);
 int notify_new_num(snac *snac);
 xs_list *notify_list(snac *snac, int skip, int show);
