@@ -8,6 +8,8 @@
 #include "xs_openssl.h"
 #include "xs_random.h"
 #include "xs_glob.h"
+#include "xs_curl.h"
+#include "xs_regex.h"
 
 #include "snac.h"
 
@@ -23,14 +25,18 @@ static const char *default_srv_config = "{"
     "\"dbglevel\":             0,"
     "\"queue_retry_minutes\":  2,"
     "\"queue_retry_max\":      10,"
+    "\"queue_timeout\":        6,"
+    "\"queue_timeout_2\":      8,"
     "\"cssurls\":              [\"\"],"
-    "\"max_timeline_entries\": 128,"
+    "\"max_timeline_entries\": 50,"
     "\"timeline_purge_days\":  120,"
     "\"local_purge_days\":     0,"
+    "\"min_account_age\":      0,"
     "\"admin_email\":          \"\","
     "\"admin_account\":        \"\","
     "\"title\":                \"\","
     "\"short_description\":    \"\","
+    "\"protocol\":             \"https\","
     "\"fastcgi\":              false"
     "}";
 
@@ -43,13 +49,14 @@ static const char *default_css =
     ".snac-top-user { text-align: center; padding-bottom: 2em }\n"
     ".snac-top-user-name { font-size: 200% }\n"
     ".snac-top-user-id { font-size: 150% }\n"
-    ".snac-avatar { float: left; height: 2.5em; padding: 0.25em }\n"
+    ".snac-announcement { border: black 1px solid; padding: 0.5em }\n"
+    ".snac-avatar { float: left; height: 2.5em; width: 2.5em; padding: 0.25em }\n"
     ".snac-author { font-size: 90%; text-decoration: none }\n"
     ".snac-author-tag { font-size: 80% }\n"
     ".snac-pubdate { color: #a0a0a0; font-size: 90% }\n"
     ".snac-top-controls { padding-bottom: 1.5em }\n"
     ".snac-post { border-top: 1px solid #a0a0a0; }\n"
-    ".snac-children { padding-left: 2em; border-left: 1px solid #a0a0a0; }\n"
+    ".snac-children { padding-left: 1em; border-left: 1px solid #a0a0a0; }\n"
     ".snac-textarea { font-family: inherit; width: 100% }\n"
     ".snac-history { border: 1px solid #606060; border-radius: 3px; margin: 2.5em 0; padding: 0 2em }\n"
     ".snac-btn-mute { float: right; margin-left: 0.5em }\n"
@@ -62,6 +69,11 @@ static const char *default_css =
     ".snac-btn-unlimit { float: right; margin-left: 0.5em }\n"
     ".snac-footer { margin-top: 2em; font-size: 75% }\n"
     ".snac-poll-result { margin-left: auto; margin-right: auto; }\n"
+    "@media (prefers-color-scheme: dark) { \n"
+    "  body, input, textarea { background-color: #000; color: #fff; }\n"
+    "  a { color: #7799dd }\n"
+    "  a:visited { color: #aa99dd }\n"
+    "}\n"
 ;
 
 const char *snac_blurb =
@@ -81,6 +93,7 @@ static const char *greeting_html =
     "<!DOCTYPE html>\n"
     "<html><head>\n"
     "<meta name=\"viewport\" content=\"width=device-width, initial-scale=1\"/>\n"
+    "<link rel=\"icon\" type=\"image/x-icon\" href=\"https://%host%/favicon.ico\"/>\n"
     "<title>Welcome to %host%</title>\n"
     "<body style=\"margin: auto; max-width: 50em\">\n"
     "%blurb%"
@@ -118,22 +131,34 @@ int snac_init(const char *basedir)
     xs *layout = xs_number_new(disk_layout);
     srv_config = xs_dict_set(srv_config, "layout", layout);
 
-    printf("Network address [%s]: ", xs_dict_get(srv_config, "address")); fflush(stdout);
-    {
-        xs *i = xs_strip_i(xs_readline(stdin));
-        if (*i)
-            srv_config = xs_dict_set(srv_config, "address", i);
-    }
+    int is_unix_socket = 0;
 
-    printf("Network port [%d]: ", (int)xs_number_get(xs_dict_get(srv_config, "port"))); fflush(stdout);
+    printf("Network address or full path to unix socket [%s]: ", xs_dict_get(srv_config, "address")); fflush(stdout);
     {
         xs *i = xs_strip_i(xs_readline(stdin));
         if (*i) {
-            xs *n = xs_number_new(atoi(i));
-            srv_config = xs_dict_set(srv_config, "port", n);
+            srv_config = xs_dict_set(srv_config, "address", i);
+
+            if (*i == '/')
+                is_unix_socket = 1;
         }
     }
 
+    if (!is_unix_socket) {
+        printf("Network port [%d]: ", (int)xs_number_get(xs_dict_get(srv_config, "port"))); fflush(stdout);
+        {
+            xs *i = xs_strip_i(xs_readline(stdin));
+            if (*i) {
+                xs *n = xs_number_new(atoi(i));
+                srv_config = xs_dict_set(srv_config, "port", n);
+            }
+        }
+    }
+    else {
+        xs *n = xs_number_new(0);
+        srv_config = xs_dict_set(srv_config, "port", n);
+    }
+
     printf("Host name: "); fflush(stdout);
     {
         xs *i = xs_strip_i(xs_readline(stdin));
@@ -346,7 +371,7 @@ void rm_rf(const char *dir)
     xs *d = xs_str_cat(xs_dup(dir), "/" "*");
     xs *l = xs_glob(d, 0, 0);
     xs_list *p = l;
-    xs_str *v;
+    const xs_str *v;
 
     if (dbglevel >= 1)
         printf("Deleting directory %s\n", dir);
@@ -381,7 +406,7 @@ int deluser(snac *user)
     int ret = 0;
     xs *fwers = following_list(user);
     xs_list *p = fwers;
-    xs_str *v;
+    const xs_str *v;
 
     while (xs_list_iter(&p, &v)) {
         xs *object = NULL;
@@ -401,3 +426,127 @@ int deluser(snac *user)
 
     return ret;
 }
+
+
+void verify_links(snac *user)
+/* verifies a user's links */
+{
+    const xs_dict *p = xs_dict_get(user->config, "metadata");
+    const char *k, *v;
+    int changed = 0;
+
+    xs *headers = xs_dict_new();
+    headers = xs_dict_append(headers, "accept", "text/html");
+    headers = xs_dict_append(headers, "user-agent", USER_AGENT " (link verify)");
+
+    int c = 0;
+    while (p && xs_dict_next(p, &k, &v, &c)) {
+        /* not an https link? skip */
+        if (!xs_startswith(v, "https:/" "/"))
+            continue;
+
+        int status;
+        xs *req = NULL;
+        xs *payload = NULL;
+        int p_size = 0;
+
+        req = xs_http_request("GET", v, headers, NULL, 0, &status,
+            &payload, &p_size, 0);
+
+        if (!valid_status(status)) {
+            snac_log(user, xs_fmt("link %s verify error %d", v, status));
+            continue;
+        }
+
+        /* extract the links */
+        xs *ls = xs_regex_select(payload, "< *(a|link) +[^>]+>");
+
+        xs_list *lp = ls;
+        const char *ll;
+        int vfied = 0;
+
+        while (!vfied && xs_list_iter(&lp, &ll)) {
+            /* extract href and rel */
+            xs *r = xs_regex_select(ll, "(href|rel) *= *(\"[^\"]*\"|'[^']*')");
+
+            /* must have both attributes */
+            if (xs_list_len(r) != 2)
+                continue;
+
+            xs *href = NULL;
+            int is_rel_me = 0;
+            xs_list *pr = r;
+            const char *ar;
+
+            while (xs_list_iter(&pr, &ar)) {
+                xs *nq = xs_dup(ar);
+
+                nq = xs_replace_i(nq, "\"", "");
+                nq = xs_replace_i(nq, "'", "");
+
+                xs *r2 = xs_split_n(nq, "=", 1);
+                if (xs_list_len(r2) != 2)
+                    continue;
+
+                xs *ak = xs_strip_i(xs_dup(xs_list_get(r2, 0)));
+                xs *av = xs_strip_i(xs_dup(xs_list_get(r2, 1)));
+
+                if (strcmp(ak, "href") == 0)
+                    href = xs_dup(av);
+                else
+                if (strcmp(ak, "rel") == 0) {
+                    /* split the value by spaces */
+                    xs *vbs = xs_split(av, " ");
+
+                    /* is any of it "me"? */
+                    if (xs_list_in(vbs, "me") != -1)
+                        is_rel_me = 1;
+                }
+            }
+
+            /* after all this acrobatics, do we have an href and a rel="me"? */
+            if (href != NULL && is_rel_me) {
+                /* is it the same as the actor? */
+                if (strcmp(href, user->actor) == 0) {
+                    /* got it! */
+                    xs *verified_time = xs_number_new((double)time(NULL));
+
+                    if (user->links == NULL)
+                        user->links = xs_dict_new();
+
+                    user->links = xs_dict_set(user->links, v, verified_time);
+
+                    vfied = 1;
+                }
+                else
+                    snac_debug(user, 1,
+                        xs_fmt("verify link %s rel='me' found but not related (%s)", v, href));
+            }
+        }
+
+        if (vfied) {
+            changed++;
+            snac_log(user, xs_fmt("link %s verified", v));
+        }
+        else {
+            snac_log(user, xs_fmt("link %s not verified (rel='me' not found)", v));
+        }
+    }
+
+    if (changed) {
+        FILE *f;
+
+        /* update the links.json file */
+        xs *fn = xs_fmt("%s/links.json", user->basedir);
+        xs *bfn = xs_fmt("%s.bak", fn);
+
+        rename(fn, bfn);
+
+        if ((f = fopen(fn, "w")) != NULL) {
+            xs_json_dump(user->links, 4, f);
+            fclose(f);
+        }
+        else
+            rename(bfn, fn);
+    }
+}
